#!/usr/local/bin/perl
#
# $Header: tfa/src/orachk/src/merge_collections.pl /main/2 2014/04/27 22:44:51 cgirdhar Exp $
#
# merge_collections.pl
#
# Copyright (c) 2013, 2014, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      merge_collections.pl - <one-line expansion of the name>
#
#    DESCRIPTION
#      <short description of component this file declares/defines>
#
#    NOTES
#      RETURN STATUS	DESCRIPTION
#   	1		Syntax Error
#    	4		Some of the Candidate Collections are faulty
#	5		Candidate Collections are not from same cluster
#
#
#    MODIFIED   (MM/DD/YY)
#    rojuyal    04/16/14 - BUG-18530680
#    rojuyal    10/24/13 - Script to merge collections
#

use warnings;
use strict;
use Getopt::Long;
use File::Basename;
use File::Copy;
use POSIX qw(strftime);
use Data::Dumper;


#Section:VARIABLES--------
my $RAW_COLLECTIONS;
my $LOCALHOST;
my $PROG_NAME;
my $BASEDIR;
my @COLLECTIONS;
my %FAULTY_COLLECTIONS;
my @NEW_COLLECTIONS;
my ($FAILCOUNT, $WARNCOUNT, $INFOCOUNT, $TOTALCOUNT) = (0, 0, 0, 0);
my @HOSTS;
my %HASH_CN;
my %HASH_CN2;
my %CLUSTER_CHK;
my %REMOVE_HASH;
my (%PROFILE, %PROFILE_NAME);
my ($STR_PROFILE, $STR_PROFILE_NAME);
my ($READ_PROF, $READ_GRP);
my %PROF;
my %RO; 


#Section:FUNCTIONS--------
sub usage {
    print 
	"Usage: $0
	    -f MERGEFILES
	    -n LOCALNODE
	    -p PROGRAM NAME
	    -d WORKING DIRECTORY
	    \n\n";
    exit(1);
}

#Main Script--------------
if ( @ARGV == 0 ) {
    usage();
}

GetOptions(
    "f=s" => \$RAW_COLLECTIONS ,
    "n=s" => \$LOCALHOST ,
    "p=s" => \$PROG_NAME ,
    "d=s" => \$BASEDIR ,
) or usage();


$BASEDIR = $BASEDIR."/";
$BASEDIR =~ s/\.cgrep//g;

my ($TARGET) = $BASEDIR.'.MERGED_COLLECTIONS/';
system("rm -rf $TARGET"); 
mkdir( $BASEDIR . '.MERGED_COLLECTIONS/' );

@COLLECTIONS = split( ',', $RAW_COLLECTIONS );

for my $inputdir (@COLLECTIONS) {
    my $node_exist = 0;
    my $abs_path;
    if ( -d $inputdir ) {
        $abs_path = $inputdir;
    }
    elsif ( -f $inputdir && $inputdir =~ m/\.zip$/ ) {
        $abs_path = $inputdir;
    }
    else {
        $abs_path = $BASEDIR . $inputdir;
        if ( !-d $abs_path ) {
	    $FAULTY_COLLECTIONS{$inputdir} = 1;
	    next;
        }
    }

    if ( $abs_path =~ m/\.zip$/ ) {
        system("unzip -o $abs_path -d $BASEDIR 1>/dev/null");
        $abs_path =~ s/\.zip//g;
    }

    $abs_path=basename($abs_path);
    $abs_path=$BASEDIR.$abs_path;
    push( @NEW_COLLECTIONS, $abs_path );

    my ($cellfile) = $abs_path . '/' . 'outfiles' . '/' . 'cell_ib_count.out';
    if ( -e $cellfile ) {
        open my $CFILE, '<', $cellfile ;
        while (<$CFILE>) {
          my $count = 0;
          if ($_ =~ m/(\d+)/) {
            $count = $1;
          }

          if ($_ =~ m/FAIL/) {
            $FAILCOUNT = $FAILCOUNT + $count;
          } elsif ($_ =~ m/WARN/) {
            $WARNCOUNT = $WARNCOUNT + $count;
          } elsif ($_ =~ m/INFO/) {
            $INFOCOUNT = $INFOCOUNT + $count;
          } elsif ($_ =~ m/TOTAL/) {     
            $TOTALCOUNT = $TOTALCOUNT + $count;
          }
        }
        close $CFILE;
    }

    my ($hfile)	= $abs_path . '/' . 'outfiles' . '/' . 'o_host_list.out';
    if ( -e $hfile ) {
        open my $HFIL, '<', $hfile; 
        my %lhost;
        while (<$HFIL>) {
            my $thost = $_;
            chomp($thost);
            $lhost{$thost} = 1;
        }
        close $HFIL;

        my %hparams = map { $_ => 1 } @HOSTS;
        for ( keys %lhost ) {
            if ( exists( $hparams{$_} ) ) {
                $node_exist = 1;
            }
            else {
                push( @HOSTS, $_ );
            }
        }
    }

    my ($chkfile) = $abs_path . '/' . 'outfiles' . '/' . 'check_env.out';
    open my $CHKFILE, '<', $chkfile;
    my $FULLTEXT = 0;
    my @text;
    my $node;
    while (<$CHKFILE>) {
        if ( $_ =~ m/CHECKED_NODE/ ) {
            my $mod_text = $_;
            $mod_text =~ s/=\s(.*$)/= $LOCALHOST/g;
            if ( exists $HASH_CN{$mod_text} ) {
                $HASH_CN{$mod_text} = $HASH_CN{$mod_text} . ":" . $1;
            }
            else {
                $HASH_CN{$mod_text} = $1;
            }
        }
        if ( $_ =~ m/^LOCALNODE/ ) {
            $node = $_;
            $node =~ s/^LOCALNODE = //g;
            chomp($node);
            if ( "$LOCALHOST" eq "$node" ) {
                $FULLTEXT = 1;
            }
        }
 
        if($_ =~ m/ROOT_OPTION/) {
		$RO{$_}=1;
	}
	else {
        	push( @text, $_ );
	}
    }
    close $CHKFILE;

    foreach( keys %HASH_CN ) {
	$HASH_CN2{$_} = $HASH_CN{$_};
    }

    if ( $node_exist == 1 ) {
	undef %HASH_CN;	
        my @ntext = ();
        foreach my $nline (@text) {
            my $match = 0;
            open my $CHKFILE, '<', $BASEDIR . '.MERGED_COLLECTIONS/check_env.out.merge';
            my ( $nkey, $nvalue ) = split( '=', $nline );
            if ( $nkey =~ m/CLUSTER_NAME/ ) {
                $CLUSTER_CHK{$nline} = 1;
            }
            if ( $nkey =~ m/PROFILE_NAMES/ ) {
                $READ_PROF = $nline;
            }
            if ( $nkey =~ m/grp_str/ ) {
                $READ_GRP = $nline;
            }
            while ( my $chkline = <$CHKFILE> ) {
                if ( $chkline eq $nline ) {
                    $match = 1;
                    last;
                }
                else {
                    my ( $chkkey, $chkvalue ) = split( '=', $chkline );
                    if ( "$nkey" eq "$chkkey" ) {
                        if ( "$chkvalue" gt "$nvalue" ) {
                            if ( "$nkey" eq 'PROFILES ' ) {
                                $REMOVE_HASH{$chkline} = 1;
                                $REMOVE_HASH{$nline} = 1;
                                $PROFILE{$nvalue}      = 1;
                                $PROFILE{$chkvalue}    = 1;
                                $match                 = 1;
                            }
                            elsif ( "$nkey" eq 'PROFILE_NAMES ' ) {
                                $REMOVE_HASH{$chkline}   = 1;
                                $REMOVE_HASH{$nline} = 1;
                                $PROFILE_NAME{$nvalue}   = 1;
                                $PROFILE_NAME{$chkvalue} = 1;
                                $match                   = 1;
                            }
                            else {
                                $match = 1;
                                last;
                            }
                        }
                        else {
                            if ( "$nkey" eq 'PROFILES ' ) {
                                $PROFILE{$nvalue}      = 1;
                                $REMOVE_HASH{$nline} = 1;
                                $PROFILE{$chkvalue}    = 1;
                            }
                            elsif ( "$nkey" eq 'PROFILE_NAMES ' ) {
                                $PROFILE_NAME{$nvalue}   = 1;
                                $REMOVE_HASH{$nline} = 1;
                                $PROFILE_NAME{$chkvalue} = 1;
                            }
			    if("$nkey" ne 'RDBMS_ORACLE_HOME ' ) {
                            	$REMOVE_HASH{$chkline} = 1;
                            	$match = 1;
			    }
                        }
                    }
		    else {
			if ($chkline =~ m/ROOT_OPTION/) {
				$RO{$chkline}=1;
				$match=1;
				last;
			}
			$match = 0;
		    }
                }
            }

            if ( $match == 0 ) {
                chomp($nline);
                push( @ntext, $nline );
            }
            close($CHKFILE);
        }

        open my $CHKFILE, '>>', $BASEDIR . '.MERGED_COLLECTIONS/check_env.out.merge';
        foreach (@ntext) {
            next if ( $_ =~ m/CHECKED_NODE = / );
            print $CHKFILE $_ . "\n";
        }
        close($CHKFILE);
    }
    else {
        open my $CHKFILE, '>>', $BASEDIR . '.MERGED_COLLECTIONS/check_env.out.merge';
        if ( $FULLTEXT == 1 ) {
            foreach (@text) {
                if ( $_ !~ m/CHECKED_NODE = / ) {
                    if ( $_ =~ m/CLUSTER_NAME = / ) {
                        $CLUSTER_CHK{$_} = 1;
                    }
                    if ( $_ =~ m/PROFILE_NAMES = / ) {
                        $READ_PROF = $_;
                    }
                    if ( $_ =~ m/grp_str = / ) {
                        $READ_GRP = $_;
                    }
                    print $CHKFILE $_;
                }
            }
        }
        else {
            foreach (@text) {
                if ( $_ =~ m/$node/ ) {
                    if (   $_ !~ m/CHECKED_NODE = /
                        && $_ !~ m/grp_str = /
                        && $_ !~ m/^LOCALNODE = /
                        && $_ !~ m/^ROOT_OPTION = /
                        && $_ !~ m/$node\.\+ASM(\d)+\.VERSION/ )
                    {
                        if ( $_ =~ m/CLUSTER_NAME = / ) {
                            $CLUSTER_CHK{$_} = 1;
                        }
                        print $CHKFILE $_;
                    }
                }
		if ( $_ =~ m/CLUSTER_NAME = / ) {
		    $CLUSTER_CHK{$_} = 1;
		}
            }
        }
        close $CHKFILE;
    }
    $PROF{$READ_PROF} = $READ_GRP if(defined $READ_PROF);
}

if (%FAULTY_COLLECTIONS) {
   my $fault_col="";
   for (keys %FAULTY_COLLECTIONS ) {
	$fault_col=$fault_col.", ".$_;
   } 
   $fault_col=~ s/^, //g;
   print "$fault_col does not exists\n";
   exit(4);
}

if (%CLUSTER_CHK) {
    my $same_cluster = scalar( keys %CLUSTER_CHK );
    if ( "$same_cluster" ne "1" ) {
        exit(5);
    }
}

my ($mchkfile) 	= $BASEDIR . '.MERGED_COLLECTIONS/check_env.out.merge'; 
my ($mhostfile) = $BASEDIR . '.MERGED_COLLECTIONS/o_host_list.out.merge';
my ($mcellfile) = $BASEDIR . '.MERGED_COLLECTIONS/cell_ib_count.out.merge';

if (%REMOVE_HASH) {
    if ( -e $mchkfile) {
        my $cur_grp_str = "";
        my $new_grp_str = "";
        open my $TMP_CHKFILE, '>>', $BASEDIR . '.MERGED_COLLECTIONS/.check_env.out.merge2';
        open my $CHKFILE, '<', $mchkfile;
        while (<$CHKFILE>) {
            if ( !exists $REMOVE_HASH{$_} ) {
                if ( $_ =~ m/grp_str =/ ) {
                    $cur_grp_str = $_;
                }
                else {
                    print $TMP_CHKFILE $_;
                }
            }
        }
        if (%PROF) {
            for ( keys %PROF ) {
                if ( $_ =~ m/dba/ ) {
                    $new_grp_str = $PROF{$_};
                }
            }
            if ($new_grp_str) {
                print $TMP_CHKFILE $new_grp_str;
            }
            else {
                print $TMP_CHKFILE $cur_grp_str;
            }
        }

        close($CHKFILE);
        close($TMP_CHKFILE);

        unlink($mchkfile); 
        rename(
            $BASEDIR . '.MERGED_COLLECTIONS/.check_env.out.merge2',
            $mchkfile 
        );
    }
}

if (%PROFILE) {
    for ( keys %PROFILE ) {
        my $temp = $_;
        chomp($temp);
        $temp =~ s/\s//g;
        $STR_PROFILE .= $temp . " ";
    }

    for ( keys %PROFILE_NAME ) {
        my $temp = $_;
        chomp($temp);
        $temp =~ s/\s//g;
        $STR_PROFILE_NAME .= $temp . ',';
    }

    open my $CHKFILE, '>>', $mchkfile;
    chop($STR_PROFILE);
    chop($STR_PROFILE_NAME);
    
    print $CHKFILE "PROFILES = " . $STR_PROFILE . "\n";
    print $CHKFILE "PROFILE_NAMES = " . $STR_PROFILE_NAME . "\n";
    close($CHKFILE);
}

if(%RO){
    my $lowest_rvalue = ( sort {$a cmp $b} keys %RO )[0];
    open my $CHKFILE, '>>', $mchkfile;
    print $CHKFILE $lowest_rvalue;
    close($CHKFILE);
}

open my $CHKFILE, '>>', $mchkfile;
foreach ( keys %HASH_CN ) {
    my $ok     = 0;
    my $DBNAME = $_;
    $DBNAME =~ s/(^.*)\.CHECKED_NODE.*$/$1/g;
    chomp($DBNAME);

    print $CHKFILE $_;
    my (@p_hosts) = split( ':', $HASH_CN{$_} );
    for my $host (@HOSTS) {
        $ok = 0;
        my $ihost = $host;
        chomp($ihost);
        for my $phost (@p_hosts) {
            my $iphost = $phost;
            if ( $ihost eq $iphost ) {
                $ok = 1;
            }
        }
        if ( $ok eq 0 ) {
            print $CHKFILE "$ihost.$DBNAME.INSTANCE_NAME = \n";
            print $CHKFILE "$ihost.$DBNAME.INSTANCE_MODE = 0\n";
            print $CHKFILE "$ihost.$DBNAME.INSTANCE_VERSION = \n";
        }
    }
}
foreach ( keys %HASH_CN2 ) {
    print $CHKFILE $_;
}
close $CHKFILE;

open my $FILE, '>>', $mhostfile;
foreach (@HOSTS) {
    print $FILE $_."\n";
}
close $FILE;

open my $CFILE, '>>', $mcellfile ;
print $CFILE "FAIL = $FAILCOUNT\n";
print $CFILE "WARN = $WARNCOUNT\n";
print $CFILE "INFO = $INFOCOUNT\n";
print $CFILE "TOTAL = $TOTALCOUNT\n";
close $CFILE;

for my $inputdir (@NEW_COLLECTIONS) {
    my ($tget) = $BASEDIR.'.MERGED_COLLECTIONS/';
    system("cp -r $inputdir $tget");
}

